# Copyright (c) 2018 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64
from unittest import mock

from castellan.common import exception
from castellan.common.objects import opaque_data

import barbican.plugin.castellan_secret_store as css
import barbican.plugin.interface.secret_store as ss
import barbican.plugin.vault_secret_store as vss
from barbican.tests import utils

key_ref1 = 'aff825be-6ede-4b1d-aeb0-aaec8e62aec6'
key_ref2 = '9c94c9c7-16ea-43e8-8ebe-0de282c0e6d5'

mock_key = b'\xae9Eso\xd4\x98\x04>\xc3\x05n\x0f\x03\x96\xa3' + \
           b'\xc3Z;\x9c\x11&oYY\x00\x13\xae\xf4>\x83\x82'


class WhenTestingVaultSecretStore(utils.BaseTestCase):

    def setUp(self):
        super(WhenTestingVaultSecretStore, self).setUp()
        self.key_manager_mock = mock.MagicMock(name="key manager mock")
        self.key_manager_mock.create_key_pair.return_value = (
            key_ref1, key_ref2
        )
        self.key_manager_mock.create_key.return_value = key_ref1
        self.key_manager_mock.store.return_value = key_ref1

        secret_object = opaque_data.OpaqueData(mock_key)
        self.key_manager_mock.get.return_value = secret_object

        self.cfg_mock = mock.MagicMock(name='config mock')
        self.cfg_mock.vault_plugin = mock.MagicMock(
            use_ssl=False,
            root_token_id='12345'
        )

        self.plugin = vss.VaultSecretStore(self.cfg_mock)
        self.plugin.key_manager = self.key_manager_mock
        self.plugin_name = "VaultSecretStore"

    def test_meta_dict(self):
        key_id = 'SOME_KEY_UUID'
        meta = self.plugin._meta_dict(key_id)
        self.assertNotIn(css.CastellanSecretStore.BIT_LENGTH, meta)
        self.assertNotIn(css.CastellanSecretStore.ALG, meta)
        self.assertEqual(key_id, meta[css.CastellanSecretStore.KEY_ID])

        meta = self.plugin._meta_dict(key_id, bit_length=128)
        self.assertEqual(128, meta[css.CastellanSecretStore.BIT_LENGTH])

        meta = self.plugin._meta_dict(key_id, algorithm='AES')
        self.assertEqual('AES', meta[css.CastellanSecretStore.ALG])

        self.assertEqual(1, meta[css.CastellanSecretStore.METADATA_VERSION])

    def test_generate_symmetric_key(self):
        key_spec = ss.KeySpec(ss.KeyAlgorithm.AES, 128)
        response = self.plugin.generate_symmetric_key(key_spec)

        self.plugin.key_manager.create_key.assert_called_once_with(
            mock.ANY,
            ss.KeyAlgorithm.AES,
            128
        )

        expected_response = {
            css.CastellanSecretStore.KEY_ID: key_ref1,
            css.CastellanSecretStore.METADATA_VERSION:
                css.CastellanSecretStore.CURRENT_VERSION}
        self.assertEqual(response, expected_response)

    def test_generate_symmetric_key_raises_exception(self):
        key_spec = ss.KeySpec(ss.KeyAlgorithm.AES, 128)
        self.plugin.key_manager.create_key.side_effect = exception.Forbidden()
        self.assertRaises(
            ss.SecretGeneralException,
            self.plugin.generate_symmetric_key,
            key_spec
        )

    def test_generate_asymmetric_key(self):
        key_spec = ss.KeySpec(ss.KeyAlgorithm.RSA, 2048)
        response = self.plugin.generate_asymmetric_key(key_spec)

        self.plugin.key_manager.create_key_pair.assert_called_once_with(
            mock.ANY,
            ss.KeyAlgorithm.RSA,
            2048)

        self.assertIsInstance(response, ss.AsymmetricKeyMetadataDTO)
        self.assertEqual(
            response.public_key_meta[css.CastellanSecretStore.KEY_ID],
            key_ref2
        )
        self.assertEqual(
            response.private_key_meta[css.CastellanSecretStore.KEY_ID],
            key_ref1
        )

    def test_generate_asymmetric_throws_exception(self):
        key_spec = ss.KeySpec(ss.KeyAlgorithm.RSA, 2048)
        self.plugin.key_manager.create_key_pair.side_effect = (
            exception.Forbidden()
        )
        self.assertRaises(
            ss.SecretGeneralException,
            self.plugin.generate_asymmetric_key,
            key_spec
        )

    def test_generate_asymmetric_throws_passphrase_exception(self):
        key_spec = ss.KeySpec(
            alg=ss.KeyAlgorithm.RSA,
            bit_length=2048,
            passphrase="some passphrase"
        )

        self.assertRaises(
            ss.GeneratePassphraseNotSupportedException,
            self.plugin.generate_asymmetric_key,
            key_spec
        )

    def test_store_secret(self):
        payload = b'encrypt me!!'
        key_spec = mock.MagicMock()
        content_type = mock.MagicMock()
        transport_key = None
        secret_dto = ss.SecretDTO(ss.SecretType.SYMMETRIC,
                                  base64.b64encode(payload),
                                  key_spec,
                                  content_type,
                                  transport_key)
        response = self.plugin.store_secret(secret_dto)

        data = opaque_data.OpaqueData(payload)
        self.plugin.key_manager.store.assert_called_once_with(
            mock.ANY,
            data
        )
        expected_response = self.plugin._meta_dict(key_ref1)
        self.assertEqual(response, expected_response)

    def test_store_secret_raises_exception(self):
        payload = b'encrypt me!!'
        key_spec = mock.MagicMock()
        content_type = mock.MagicMock()
        transport_key = None
        secret_dto = ss.SecretDTO(ss.SecretType.SYMMETRIC,
                                  base64.b64encode(payload),
                                  key_spec,
                                  content_type,
                                  transport_key)

        self.plugin.key_manager.store.side_effect = exception.Forbidden()
        self.assertRaises(
            ss.SecretGeneralException,
            self.plugin.store_secret,
            secret_dto
        )

    def test_get_secret(self):
        secret_metadata = self.plugin._meta_dict(key_ref1, 256, 'AES')

        response = self.plugin.get_secret(
            ss.SecretType.SYMMETRIC,
            secret_metadata
        )

        self.assertIsInstance(response, ss.SecretDTO)

        plaintext = base64.b64decode(response.secret)

        self.assertEqual(ss.SecretType.SYMMETRIC, response.type)
        self.assertEqual(mock_key, plaintext)
        self.plugin.key_manager.get.assert_called_once_with(
            mock.ANY,
            key_ref1
        )

    def test_get_secret_throws_exception(self):
        secret_metadata = self.plugin._meta_dict(key_ref1, 256, 'AES')
        self.plugin.key_manager.get.side_effect = exception.Forbidden()
        self.assertRaises(
            ss.SecretGeneralException,
            self.plugin.get_secret,
            ss.SecretType.SYMMETRIC,
            secret_metadata
        )

    def test_delete_secret(self):
        secret_metadata = {css.CastellanSecretStore.KEY_ID: key_ref1}
        self.plugin.delete_secret(secret_metadata)
        self.plugin.key_manager.delete.assert_called_once_with(
            mock.ANY,
            key_ref1
        )

    def test_delete_secret_throws_exception(self):
        secret_metadata = {css.CastellanSecretStore.KEY_ID: key_ref1}
        self.plugin.key_manager.delete.side_effect = exception.Forbidden()
        self.assertRaises(
            ss.SecretGeneralException,
            self.plugin.delete_secret,
            secret_metadata
        )

    def test_delete_secret_throws_key_error(self):
        secret_metadata = {css.CastellanSecretStore.KEY_ID: key_ref1}
        self.plugin.key_manager.delete.side_effect = KeyError()
        self.plugin.delete_secret(secret_metadata)
        self.plugin.key_manager.delete.assert_called_once_with(
            mock.ANY,
            key_ref1
        )

    def test_store_secret_supports(self):
        self.assertTrue(
            self.plugin.generate_supports(mock.ANY)
        )

    def test_generate_supports(self):
        self.assertTrue(
            self.plugin.generate_supports(mock.ANY)
        )

    def test_get_plugin_name(self):
        self.assertEqual(self.plugin_name, self.plugin.get_plugin_name())



#!/usr/bin/env python

# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import argparse
import base64
import traceback

from oslo_db.sqlalchemy import session
from oslo_serialization import jsonutils as json
from sqlalchemy import orm
from sqlalchemy.orm import scoping

from barbican.common import utils
from barbican.model import models
from barbican.plugin.crypto import p11_crypto
from barbican.plugin.crypto.pkcs11 import P11CryptoPluginException

# Use config values from p11_crypto
CONF = p11_crypto.CONF


class KekSignatureMigrator(object):

    def __init__(self, db_connection, library_path, login, slot_id):
        self.dry_run = False
        self.db_engine = session.create_engine(db_connection)
        self._session_creator = scoping.scoped_session(
            orm.sessionmaker(
                bind=self.db_engine,
                autocommit=True
            )
        )
        self.crypto_plugin = p11_crypto.P11CryptoPlugin(CONF)
        self.plugin_name = utils.generate_fullname_for(self.crypto_plugin)
        self.pkcs11 = self.crypto_plugin.pkcs11
        self.session = self.pkcs11.get_session()

    def recalc_kek_hmac(self, project, kek):
        with self.db_session.begin():
            meta_dict = json.loads(kek.plugin_meta)
            iv = base64.b64decode(meta_dict['iv'])
            wrapped_key = base64.b64decode(meta_dict['wrapped_key'])
            hmac = base64.b64decode(meta_dict['hmac'])
            kek_data = iv + wrapped_key
            hmac_key = self.pkcs11.get_key_handle(
                meta_dict['hmac_label'], self.session)

            # Verify if hmac signature validates with new method
            try:
                self.pkcs11.verify_hmac(hmac_key, hmac, kek_data, self.session)
                sig_good = True
            except P11CryptoPluginException as e:
                if 'CKR_SIGNATURE_INVALID' in str(e):
                    sig_good = False
                else:
                    raise

            if sig_good:
                msg = 'Skipping KEK {}, good signature'
                print(msg.format(kek.kek_label))
                return

            # Previous method failed.
            # Verify if hmac signature validates with old method
            try:
                self.pkcs11.verify_hmac(
                    hmac_key, hmac, wrapped_key, self.session
                )
                sig_bad = True
            except P11CryptoPluginException as e:
                if 'CKR_SIGNATURE_INVALID' in str(e):
                    sig_bad = False
                else:
                    raise

            if not sig_bad:
                msg = "Skipping KEK {}, can not validate with either method!"
                print(msg.format(kek.kek_label))
                return

            if self.dry_run:
                msg = 'KEK {} needs recalculation'
                print(msg.format(kek.kek_label))
                return

            # Calculate new HMAC
            new_hmac = self.pkcs11.compute_hmac(
                hmac_key, kek_data, self.session
            )

            # Update KEK plugin_meta with new hmac signature
            meta_dict['hmac'] = base64.b64encode(new_hmac)
            kek.plugin_meta = p11_crypto.json_dumps_compact(meta_dict)

    def get_keks_for_project(self, project):
        keks = []
        with self.db_session.begin() as transaction:
            print('Retrieving KEKs for Project {}'.format(project.id))
            query = transaction.session.query(models.KEKDatum)
            query = query.filter_by(project_id=project.id)
            query = query.filter_by(plugin_name=self.plugin_name)

            keks = query.all()

        return keks

    def get_projects(self):
        print('Retrieving all available projects')

        projects = []
        with self.db_session.begin() as transaction:
            projects = transaction.session.query(models.Project).all()

        return projects

    @property
    def db_session(self):
        return self._session_creator()

    def execute(self, dry_run=True):
        self.dry_run = dry_run
        if self.dry_run:
            print('-- Running in dry-run mode --')

        projects = self.get_projects()
        for project in projects:
            keks = self.get_keks_for_project(project)
            for kek in keks:
                try:
                    self.recalc_kek_hmac(project, kek)
                except Exception:
                    print('Error occurred! SQLAlchemy automatically rolled-'
                          'back the transaction')
                    traceback.print_exc()


def main():
    script_desc = (
        'Utility to migrate existing project KEK signatures to include IV.'
    )

    parser = argparse.ArgumentParser(description=script_desc)
    parser.add_argument(
        '--dry-run',
        action='store_true',
        help='Displays changes that will be made (Non-destructive)'
    )
    args = parser.parse_args()

    migrator = KekSignatureMigrator(
        db_connection=CONF.sql_connection,
        library_path=CONF.p11_crypto_plugin.library_path,
        login=CONF.p11_crypto_plugin.login,
        slot_id=CONF.p11_crypto_plugin.slot_id
    )
    migrator.execute(args.dry_run)


if __name__ == '__main__':
    main()



# Copyright 2018 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from webob import acceptparse


if hasattr(acceptparse, 'create_accept_header'):
    # WebOb >= 1.8.0
    NoHeaderType = getattr(acceptparse, 'AcceptNoHeader')
    ValidHeaderType = getattr(acceptparse, 'AcceptValidHeader')
    create_accept_header = getattr(acceptparse, 'create_accept_header')
else:
    # WebOb < 1.8.0
    NoHeaderType = getattr(acceptparse, 'MIMENilAccept')
    ValidHeaderType = getattr(acceptparse, 'MIMEAccept')

    def create_accept_header(header_value):
        if not header_value:
            return NoHeaderType()
        else:
            return ValidHeaderType(header_value)



#!/usr/bin/env python
# Copyright 2010-2015 OpenStack LLC.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import os
import sys

sys.path.insert(0, os.getcwd())

from barbican.common import config
from barbican.model import clean
from barbican.model.migration import commands
from oslo_log import log

# Import and configure logging.
CONF = config.CONF
log.setup(CONF, 'barbican')
LOG = log.getLogger(__name__)


class DatabaseManager(object):
    """Database Manager class.

    Builds and executes a CLI parser to manage the Barbican database
    This extends the Alembic commands.
    """

    def __init__(self, conf):
        self.conf = conf
        self.parser = self.get_main_parser()
        self.subparsers = self.parser.add_subparsers(
            title='subcommands',
            description='Action to perform')
        self.add_revision_args()
        self.add_upgrade_args()
        self.add_history_args()
        self.add_current_args()
        self.add_clean_args()

    def get_main_parser(self):
        """Create top-level parser and arguments."""
        parser = argparse.ArgumentParser(description='Barbican DB manager.')
        parser.add_argument('--dburl', '-d', default=self.conf.sql_connection,
                            help='URL to the database.')

        return parser

    def add_revision_args(self):
        """Create 'revision' command parser and arguments."""
        create_parser = self.subparsers.add_parser('revision', help='Create a '
                                                   'new DB version file.')
        create_parser.add_argument('--message', '-m', default='DB change',
                                   help='the message for the DB change')
        create_parser.add_argument('--autogenerate',
                                   help='autogenerate from models',
                                   action='store_true')
        create_parser.set_defaults(func=self.revision)

    def add_upgrade_args(self):
        """Create 'upgrade' command parser and arguments."""
        create_parser = self.subparsers.add_parser('upgrade',
                                                   help='Upgrade to a '
                                                   'future version DB '
                                                   'version file')
        create_parser.add_argument('--version', '-v', default='head',
                                   help='the version to upgrade to, or else '
                                        'the latest/head if not specified.')
        create_parser.set_defaults(func=self.upgrade)

    def add_history_args(self):
        """Create 'history' command parser and arguments."""
        create_parser = self.subparsers.add_parser(
            'history',
            help='List changeset scripts in chronological order.')
        create_parser.add_argument('--verbose', '-V', action="store_true",
                                   help='Show full information about the '
                                        'revisions.')
        create_parser.set_defaults(func=self.history)

    def add_current_args(self):
        """Create 'current' command parser and arguments."""
        create_parser = self.subparsers.add_parser(
            'current',
            help='Display the current revision for a database.')
        create_parser.add_argument('--verbose', '-V', action="store_true",
                                   help='Show full information about the '
                                        'revision.')
        create_parser.set_defaults(func=self.current)

    def add_clean_args(self):
        """Create 'clean' command parser and arguments."""
        create_parser = self.subparsers.add_parser(
            'clean',
            help='Clean up soft deletions in the database')
        create_parser.add_argument(
            '--min-days', '-m', type=int, default=90,
            help='minimum number of days to keep soft deletions. default is'
                 ' %(default)s days.')
        create_parser.add_argument('--clean-unassociated-projects', '-p',
                                   action="store_true",
                                   help='Remove projects that have no '
                                        'associated resources.')
        create_parser.add_argument('--soft-delete-expired-secrets', '-e',
                                   action="store_true",
                                   help='Soft delete expired secrets.')
        create_parser.add_argument('--verbose', '-V', action='store_true',
                                   help='Show full information about the'
                                        ' cleanup')
        create_parser.add_argument('--log-file', '-L',
                                   default=CONF.log_file,
                                   type=str,
                                   help='Set log file location. '
                                        'Default value for log_file can be '
                                        'found in barbican.conf')
        create_parser.set_defaults(func=self.clean)

    def revision(self, args):
        """Process the 'revision' Alembic command."""
        commands.generate(autogenerate=args.autogenerate,
                          message=args.message,
                          sql_url=args.dburl)

    def upgrade(self, args):
        """Process the 'upgrade' Alembic command."""
        LOG.debug("Performing database schema migration...")
        commands.upgrade(to_version=args.version, sql_url=args.dburl)

    def history(self, args):
        commands.history(args.verbose, sql_url=args.dburl)

    def current(self, args):
        commands.current(args.verbose, sql_url=args.dburl)

    def clean(self, args):
        clean.clean_command(
            sql_url=args.dburl,
            min_num_days=args.min_days,
            do_clean_unassociated_projects=args.clean_unassociated_projects,
            do_soft_delete_expired_secrets=args.soft_delete_expired_secrets,
            verbose=args.verbose,
            log_file=args.log_file)

    def execute(self):
        """Parse the command line arguments."""
        args = self.parser.parse_args()

        # Perform other setup here...

        args.func(args)


def _exception_is_successful_exit(thrown_exception):
    return (isinstance(thrown_exception, SystemExit) and
            (thrown_exception.code is None or thrown_exception.code == 0))


def main():

    try:
        dm = DatabaseManager(CONF)
        dm.execute()
    except Exception as ex:
        if not _exception_is_successful_exit(ex):
            LOG.exception('Problem seen trying to run barbican db manage')
            sys.stderr.write("ERROR: {0}\n".format(ex))
            sys.exit(1)


if __name__ == '__main__':
    main()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from barbican.common import exception
from barbican.model import models
from barbican.model import repositories
from barbican.tests import database_utils


class TestACLMixin(object):

    def _assert_acl_users(self, user_ids, acls, acl_id, check_size=True):
        """Checks that all input users are present in matching acl users data.

        It also checks if number of acl users are same as input users when
        check_size flag is True.
        """
        acls_map = self._map_id_to_acl(acls)
        acl_users = acls_map[acl_id].to_dict_fields()['users']
        if check_size:
            self.assertEqual(len(user_ids), len(acl_users))
        self.assertTrue(all(user_id in user_ids for user_id in acl_users))

    def _map_id_to_acl(self, acls):
        """Provides dictionary of id and acl from acls list."""
        m = {}
        for acl in acls:
            m[acl.id] = acl
        return m


class WhenTestingSecretACLRepository(database_utils.RepositoryTestCase,
                                     TestACLMixin):

    def setUp(self):
        super(WhenTestingSecretACLRepository, self).setUp()
        self.acl_repo = repositories.get_secret_acl_repository()

    def _create_base_secret(self, project_id=None):
        # Setup the secret and needed base relationship
        secret_repo = repositories.get_secret_repository()
        session = secret_repo.get_session()

        if project_id is None:  # don't re-create project if it created earlier
            project = models.Project()
            project.external_id = "keystone_project_id"
            project.save(session=session)
            project_id = project.id

        secret_model = models.Secret()
        secret_model.project_id = project_id
        secret = secret_repo.create_from(secret_model, session=session)

        secret.save(session=session)

        session.commit()
        return secret

    def test_get_by_secret_id(self):
        session = self.acl_repo.get_session()
        secret = self._create_base_secret()

        acls = self.acl_repo.get_by_secret_id(secret.id, session)
        self.assertEqual(0, len(acls))

        acl1 = self.acl_repo.create_from(models.SecretACL(secret.id, 'read',
                                                          True, ['u1', 'u2']),
                                         session)
        acls = self.acl_repo.get_by_secret_id(secret.id, session)

        self.assertEqual(1, len(acls))
        self.assertEqual(acl1.id, acls[0].id)
        self.assertEqual('read', acls[0].operation)
        self._assert_acl_users(['u2', 'u1'], acls, acl1.id)

    def test_get_by_entity_id(self):
        session = self.acl_repo.get_session()
        secret = self._create_base_secret()

        acl1 = self.acl_repo.create_from(models.SecretACL(
            secret.id, 'read', True, ['u1', 'u2']), session)
        acl = self.acl_repo.get(acl1.id, session)
        self.assertIsNotNone(acl)
        self.assertEqual(acl1.id, acl.id)
        self.assertEqual('read', acl.operation)
        self._assert_acl_users(['u1', 'u2'], [acl], acl1.id)

        self.acl_repo.delete_entity_by_id(acl1.id, session)
        acl = self.acl_repo.get(acl1.id, session, suppress_exception=True)
        self.assertIsNone(acl)

    def test_should_raise_notfound_exception_get_by_entity_id(self):
        self.assertRaises(exception.NotFound, self.acl_repo.get,
                          "invalid_id", suppress_exception=False)

    def test_create_or_replace_from_for_new_acls(self):
        """Check create_or_replace_from and get count call.

        It creates new acls with users and make sure that same users
        are returned when acls are queries by secret id.
        It uses get count to assert expected number of acls for that secret.
        """
        session = self.acl_repo.get_session()
        secret = self._create_base_secret()
        acl1 = self.acl_repo.create_from(models.SecretACL(
            secret.id, 'read'), session)
        self.acl_repo.create_or_replace_from(
            secret, acl1, user_ids=['u1', 'u2'], session=session)

        acl2 = self.acl_repo.create_from(models.SecretACL(
            secret.id, 'write', False), session)
        self.acl_repo.create_or_replace_from(
            secret, acl2, user_ids=['u1', 'u2', 'u3'], session=session)

        acl3 = self.acl_repo.create_from(models.SecretACL(
            secret.id, 'delete'), session)
        self.acl_repo.create_or_replace_from(
            secret, acl3, user_ids=[], session=session)

        acls = self.acl_repo.get_by_secret_id(secret.id, session)

        self.assertEqual(3, len(acls))

        id_map = self._map_id_to_acl(acls)
        self.assertTrue(id_map[acl1.id].project_access)
        self.assertFalse(id_map[acl2.id].project_access)
        self.assertEqual('read', id_map[acl1.id].operation)
        self.assertEqual('write', id_map[acl2.id].operation)
        self.assertEqual('delete', id_map[acl3.id].operation)
        # order of input users should not matter
        self._assert_acl_users(['u1', 'u2'], acls, acl1.id)
        self._assert_acl_users(['u2', 'u1'], acls, acl1.id)
        self._assert_acl_users(['u2', 'u1', 'u3'], acls, acl2.id)

        count = self.acl_repo.get_count(secret.id, session)
        self.assertEqual(3, count)
        self.assertEqual(count, len(acls))

    def test_create_or_replace_from_with_none_or_blank_users(self):
        session = self.acl_repo.get_session()
        secret = self._create_base_secret()
        acl1 = self.acl_repo.create_from(models.SecretACL(
            secret.id, 'read'), session)
        self.acl_repo.create_or_replace_from(
            secret, acl1, user_ids=None, session=session)

        acl2 = self.acl_repo.create_from(models.SecretACL(
            secret.id, 'list'), session)
        self.acl_repo.create_or_replace_from(
            secret, acl1, user_ids=[], session=session)

        acls = self.acl_repo.get_by_secret_id(secret.id, session)
        id_map = self._map_id_to_acl(acls)
        self.assertIsNone(id_map[acl1.id].to_dict_fields().get('users'))
        self.assertIsNone(id_map[acl2.id].to_dict_fields().get('users'))

    def test_create_or_replace_from_for_existing_acls(self):
        """Check create_or_replace_from and get count call.

        It modifies existing acls with users and make sure that updated users
        and project_access flag changes are returned when acls are queries by
        secret id. It uses get count to assert expected number of acls for that
        secret.
        """
        session = self.acl_repo.get_session()
        secret = self._create_base_secret()
        acl1 = self.acl_repo.create_from(models.SecretACL(
            secret.id, 'read'), session)
        self.acl_repo.create_or_replace_from(
            secret, acl1, user_ids=['u1', 'u2'], session=session)

        acl2 = self.acl_repo.create_from(models.SecretACL(
            secret.id, 'write'), session)
        self.acl_repo.create_or_replace_from(
            secret, acl2, user_ids=['u1', 'u2', 'u3'], session=session)

        acl3 = self.acl_repo.create_from(models.SecretACL(
            secret.id, 'list'), session)
        self.acl_repo.create_or_replace_from(
            secret, acl3, user_ids=[], session=session)

        acls = self.acl_repo.get_by_secret_id(secret.id, session)

        self.assertEqual(3, len(acls))

        id_map = self._map_id_to_acl(acls)
        # replace users in existing acls
        id_map[acl1.id].project_access = False
        self.acl_repo.create_or_replace_from(
            secret, id_map[acl1.id], user_ids=['u5'], session=session)

        self.acl_repo.create_or_replace_from(
            secret, id_map[acl2.id], user_ids=['u1', 'u2', 'u3', 'u4'],
            session=session)

        self.acl_repo.create_or_replace_from(
            secret, id_map[acl3.id], user_ids=['u1', 'u2', 'u4'],
            session=session)

        session.commit()  # commit the changes made so far
        acls = self.acl_repo.get_by_secret_id(secret.id, session)
        id_map = self._map_id_to_acl(acls)

        self.assertEqual(3, len(acls))
        self.assertFalse(id_map[acl1.id].project_access)
        self.assertTrue(id_map[acl2.id].project_access)
        self.assertTrue(id_map[acl3.id].project_access)
        self._assert_acl_users(['u5'], acls, acl1.id)
        self._assert_acl_users(['u1', 'u2', 'u3', 'u4'], acls, acl2.id)
        self._assert_acl_users(['u1', 'u2', 'u4'], acls, acl3.id)

    def test_get_count(self):
        session = self.acl_repo.get_session()
        secret1 = self._create_base_secret()
        acl1 = self.acl_repo.create_from(models.SecretACL(secret1.id, 'read',
                                                          None, ['u1', 'u2']),
                                         session)
        self.acl_repo.create_or_replace_from(secret1, acl1)

        secret2 = self._create_base_secret(secret1.project.id)
        acl21 = self.acl_repo.create_from(models.SecretACL(secret2.id, 'read',
                                                           None, ['u3', 'u4']),
                                          session)
        self.acl_repo.create_or_replace_from(secret2, acl21)
        acl22 = self.acl_repo.create_from(models.SecretACL(secret2.id, 'write',
                                                           None, ['u5', 'u6']),
                                          session)
        self.acl_repo.create_or_replace_from(secret2, acl22)

        self.assertEqual(1, self.acl_repo.get_count(secret1.id))
        self.assertEqual(2, self.acl_repo.get_count(secret2.id))

    def test_delete_single_acl_and_count(self):

        session = self.acl_repo.get_session()
        secret = self._create_base_secret()
        acl1 = self.acl_repo.create_from(models.SecretACL(secret.id, 'read',
                                                          None, ['u1', 'u2']),
                                         session)
        self.acl_repo.create_or_replace_from(secret, acl1)
        acl2 = self.acl_repo.create_from(
            models.SecretACL(secret.id, 'write'), session)
        self.acl_repo.create_or_replace_from(
            secret, acl2, user_ids=['u1', 'u2', 'u3'])
        acl3 = self.acl_repo.create_from(models.SecretACL(
            secret.id, 'list'), session)
        self.acl_repo.create_or_replace_from(secret, acl3,
                                             user_ids=['u1', 'u3'])

        count = self.acl_repo.get_count(secret.id)
        self.assertEqual(3, count)

        self.acl_repo.delete_entity_by_id(acl2.id, None)
        session.commit()
        self.assertEqual(2, len(secret.secret_acls))

        deleted_acl = self.acl_repo.get(acl2.id, suppress_exception=True)
        self.assertIsNone(deleted_acl)

        acls = self.acl_repo.get_by_secret_id(secret.id)
        self.assertEqual(2, len(acls))

        count = self.acl_repo.get_count(secret.id)
        self.assertEqual(2, count)

    def test_delete_acls_for_secret(self):
        session = self.acl_repo.get_session()
        secret = self._create_base_secret()
        acl1 = self.acl_repo.create_from(models.SecretACL(
            secret.id, 'read'), session)
        self.acl_repo.create_or_replace_from(
            secret, acl1, user_ids=['u1', 'u2'], session=session)
        acl2 = self.acl_repo.create_from(models.SecretACL(
            secret.id, 'write'), session)
        self.acl_repo.create_or_replace_from(
            secret, acl2, user_ids=['u1', 'u2', 'u3'], session=session)

        self.acl_repo.delete_acls_for_secret(secret)
        acls = self.acl_repo.get_by_secret_id(secret.id)
        self.assertEqual(0, len(acls))


class WhenTestingContainerACLRepository(database_utils.RepositoryTestCase,
                                        TestACLMixin):

    def setUp(self):
        super(WhenTestingContainerACLRepository, self).setUp()
        self.acl_repo = repositories.get_container_acl_repository()

    def _create_base_container(self, project_id=None):
        # Setup the container and needed base relationship
        container_repo = repositories.get_container_repository()
        session = container_repo.get_session()

        if project_id is None:
            project = models.Project()
            project.external_id = "keystone_project_id"
            project.save(session=session)
            project_id = project.id

        container = models.Container()

        container.project_id = project_id
        container.save(session=session)

        session.commit()
        return container

    def test_get_by_container_id(self):
        session = self.acl_repo.get_session()
        container = self._create_base_container()

        acls = self.acl_repo.get_by_container_id(container.id, session)
        self.assertEqual(0, len(acls))

        acl1 = self.acl_repo.create_from(models.ContainerACL(container.id,
                                                             'read', True,
                                                             ['u1', 'u2']),
                                         session)
        acls = self.acl_repo.get_by_container_id(container.id, session)
        self.assertEqual(1, len(acls))
        self.assertEqual(acl1.id, acls[0].id)
        self.assertEqual('read', acls[0].operation)
        self._assert_acl_users(['u1', 'u2'], acls, acl1.id)

    def test_get_by_entity_id(self):
        session = self.acl_repo.get_session()
        container = self._create_base_container()

        acl1 = self.acl_repo.create_from(models.ContainerACL(
            container.id, 'read', True, ['u1', 'u2']), session)

        acl = self.acl_repo.get(acl1.id, session)
        self.assertIsNotNone(acl)
        self.assertEqual(acl1.id, acl.id)
        self.assertEqual('read', acl.operation)
        self._assert_acl_users(['u1', 'u2'], [acl], acl1.id)

        self.acl_repo.delete_entity_by_id(acl1.id, session)
        acl = self.acl_repo.get(acl1.id, session, suppress_exception=True)
        self.assertIsNone(acl)

    def test_should_raise_notfound_exception_get_by_entity_id(self):
        self.assertRaises(exception.NotFound, self.acl_repo.get,
                          "invalid_id", suppress_exception=False)

    def test_create_or_replace_from_for_new_acls(self):
        """Check create_or_replace_from and get count call.

        It creates new acls with users and make sure that same users
        are returned when acls are queries by secret id.
        It uses get count to assert expected number of acls for that secret.
        """
        session = self.acl_repo.get_session()
        container = self._create_base_container()
        acl1 = self.acl_repo.create_from(models.ContainerACL(
            container.id, 'read'), session)
        self.acl_repo.create_or_replace_from(
            container, acl1, user_ids=['u1', 'u2'], session=session)

        acl2 = self.acl_repo.create_from(models.ContainerACL(
            container.id, 'write', False), session)
        self.acl_repo.create_or_replace_from(
            container, acl2, user_ids=['u1', 'u2', 'u3'], session=session)

        acl3 = self.acl_repo.create_from(models.ContainerACL(
            container.id, 'list'), session)
        self.acl_repo.create_or_replace_from(
            container, acl3, user_ids=[], session=session)

        acls = self.acl_repo.get_by_container_id(container.id, session)

        self.assertEqual(3, len(acls))

        id_map = self._map_id_to_acl(acls)
        self.assertTrue(id_map[acl1.id].project_access)
        self.assertFalse(id_map[acl2.id].project_access)
        self.assertEqual('read', id_map[acl1.id].operation)
        self.assertEqual('write', id_map[acl2.id].operation)
        self.assertEqual('list', id_map[acl3.id].operation)
        # order of input users should not matter
        self._assert_acl_users(['u1', 'u2'], acls, acl1.id)
        self._assert_acl_users(['u2', 'u1'], acls, acl1.id)
        self._assert_acl_users(['u2', 'u1', 'u3'], acls, acl2.id)

        count = self.acl_repo.get_count(container.id, session)
        self.assertEqual(3, count)
        self.assertEqual(count, len(acls))

    def test_create_or_replace_from_with_none_or_blank_users(self):
        session = self.acl_repo.get_session()
        container = self._create_base_container()
        acl1 = self.acl_repo.create_from(models.ContainerACL(
            container.id, 'read'), session)
        self.acl_repo.create_or_replace_from(
            container, acl1, user_ids=None, session=session)

        acl2 = self.acl_repo.create_from(models.ContainerACL(
            container.id, 'write'), session)
        self.acl_repo.create_or_replace_from(
            container, acl1, user_ids=[], session=session)

        acls = self.acl_repo.get_by_container_id(container.id, session)
        id_map = self._map_id_to_acl(acls)
        self.assertIsNone(id_map[acl1.id].to_dict_fields().get('users'))
        self.assertIsNone(id_map[acl2.id].to_dict_fields().get('users'))

    def test_create_or_replace_from_for_existing_acls(self):
        """Check create_or_replace_from and get count call.

        It modifies existing acls with users and make sure that updated users
        and project_access flag changes are returned when acls are queries by
        secret id. It uses get count to assert expected number of acls for that
        secret.
        """
        session = self.acl_repo.get_session()
        container = self._create_base_container()
        acl1 = self.acl_repo.create_from(models.ContainerACL(
            container.id, 'read'), session)
        self.acl_repo.create_or_replace_from(
            container, acl1, user_ids=['u1', 'u2'], session=session)

        acl2 = self.acl_repo.create_from(models.ContainerACL(
            container.id, 'write'), session)
        self.acl_repo.create_or_replace_from(
            container, acl2, user_ids=['u1', 'u2', 'u3'], session=session)

        acl3 = self.acl_repo.create_from(models.ContainerACL(
            container.id, 'list'), session)
        self.acl_repo.create_or_replace_from(
            container, acl3, user_ids=[], session=session)

        acls = self.acl_repo.get_by_container_id(container.id, session)

        self.assertEqual(3, len(acls))

        id_map = self._map_id_to_acl(acls)
        # replace users in existing acls
        id_map[acl1.id].project_access = False
        self.acl_repo.create_or_replace_from(
            container, id_map[acl1.id], user_ids=['u5'], session=session)

        self.acl_repo.create_or_replace_from(
            container, id_map[acl2.id], user_ids=['u1', 'u2', 'u3', 'u4'],
            session=session)

        self.acl_repo.create_or_replace_from(
            container, id_map[acl3.id], user_ids=['u1', 'u2', 'u4'],
            session=session)

        session.commit()
        acls = self.acl_repo.get_by_container_id(container.id, session)
        id_map = self._map_id_to_acl(acls)

        self.assertEqual(3, len(acls))
        self.assertFalse(id_map[acl1.id].project_access)
        self.assertTrue(id_map[acl2.id].project_access)
        self.assertTrue(id_map[acl3.id].project_access)
        self._assert_acl_users(['u5'], acls, acl1.id)
        self._assert_acl_users(['u1', 'u2', 'u3', 'u4'], acls, acl2.id)
        self._assert_acl_users(['u1', 'u2', 'u4'], acls, acl3.id)

    def test_get_count(self):
        session = self.acl_repo.get_session()
        container1 = self._create_base_container()
        acl1 = self.acl_repo.create_from(models.ContainerACL(
            container1.id, 'read', None, ['u1', 'u2']), session)
        self.acl_repo.create_or_replace_from(container1, acl1)

        container2 = self._create_base_container(container1.project_id)
        acl21 = self.acl_repo.create_from(models.ContainerACL(
            container2.id, 'read', None, ['u3', 'u4']), session)
        self.acl_repo.create_or_replace_from(container2, acl21)
        acl22 = self.acl_repo.create_from(models.ContainerACL(
            container2.id, 'write', None, ['u5', 'u6']), session)
        self.acl_repo.create_or_replace_from(container2, acl22)

        self.assertEqual(1, self.acl_repo.get_count(container1.id))
        self.assertEqual(2, self.acl_repo.get_count(container2.id))

    def test_delete_single_acl_and_count(self):

        session = self.acl_repo.get_session()
        container = self._create_base_container()
        acl1 = self.acl_repo.create_from(models.ContainerACL(
            container.id, 'read'), session)
        self.acl_repo.create_or_replace_from(container, acl1,
                                             user_ids=['u1', 'u2'])
        acl2 = self.acl_repo.create_from(models.ContainerACL(
            container.id, 'write'), session)
        self.acl_repo.create_or_replace_from(container, acl2,
                                             user_ids=['u1', 'u2', 'u3'])
        acl3 = self.acl_repo.create_from(models.ContainerACL(
            container.id, 'list'), session)
        self.acl_repo.create_or_replace_from(container, acl3,
                                             user_ids=['u1', 'u3'])

        count = self.acl_repo.get_count(container.id)
        self.assertEqual(3, count)

        self.acl_repo.delete_entity_by_id(acl2.id, None)
        session.commit()  # commit the changes made so far
        self.assertEqual(2, len(container.container_acls))

        deleted_acl = self.acl_repo.get(acl2.id, suppress_exception=True)
        self.assertIsNone(deleted_acl)

        acls = self.acl_repo.get_by_container_id(container.id)
        self.assertEqual(2, len(acls))

        count = self.acl_repo.get_count(container.id)
        self.assertEqual(2, count)

    def test_delete_acls_for_secret(self):
        session = self.acl_repo.get_session()
        container = self._create_base_container()
        acl1 = self.acl_repo.create_from(models.ContainerACL(
            container.id, 'read'), session)
        self.acl_repo.create_or_replace_from(
            container, acl1, user_ids=['u1', 'u2'], session=session)
        acl2 = self.acl_repo.create_from(models.ContainerACL(
            container.id, 'write'), session)
        self.acl_repo.create_or_replace_from(
            container, acl2, user_ids=['u1', 'u2', 'u3'], session=session)

        self.acl_repo.delete_acls_for_container(container)
        acls = self.acl_repo.get_by_container_id(container.id)
        self.assertEqual(0, len(acls))



#    Copyright 2018 Fujitsu.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_versionedobjects import base as object_base

from barbican.model import models
from barbican.model import repositories as repos
from barbican.objects import base
from barbican.objects import container_secret as con_se
from barbican.objects import fields

TYPE_VALUE = ['generic', 'rsa', 'dsa', 'certificate']


@object_base.VersionedObjectRegistry.register
class Container(base.BarbicanObject, base.BarbicanPersistentObject,
                object_base.VersionedObjectDictCompat):
    fields = {
        'name': fields.StringField(nullable=True, default=None),
        'type': fields.EnumField(nullable=True, valid_values=TYPE_VALUE),
        'project_id': fields.StringField(nullable=True, default=None),
        'creator_id': fields.StringField(nullable=True, default=None),
        'consumers': fields.ListOfObjectsField('ContainerConsumerMetadatum',
                                               default=list()),
        'container_secrets': fields.ListOfObjectsField('ContainerSecret',
                                                       default=list()),
        'container_acls': fields.ListOfObjectsField('ContainerACL',
                                                    default=list()),
        'project': fields.ObjectField('Project', nullable=True, default=None)
    }

    db_model = models.Container
    db_repo = repos.get_container_repository()
    synthetic_fields = ['consumers', 'container_secrets',
                        'container_acls', 'project']

    def __init__(self, context=None, parsed_request=None, **kwargs):
        super(Container, self).__init__(context=context, **kwargs)
        if parsed_request:
            self.name = parsed_request.get('name')
            self.type = parsed_request.get('type')
            self.status = base.States.ACTIVE
            self.creator_id = parsed_request.get('creator_id')

            secret_refs = parsed_request.get('secret_refs')
            if secret_refs:
                for secret_ref in parsed_request.get('secret_refs'):
                    container_secret = con_se.ContainerSecret()
                    container_secret.name = secret_ref.get('name')
                    secret_id = secret_ref.get('secret_ref')
                    if secret_id.endswith('/'):
                        secret_id = secret_id.rsplit('/', 2)[1]
                    elif '/' in secret_id:
                        secret_id = secret_id.rsplit('/', 1)[1]
                    else:
                        secret_id = secret_id
                    container_secret.secret_id = secret_id
                    self.container_secrets.append(container_secret)

    def _get_db_entity(self, data=None):
        return self.db_model(parsed_request=data, check_exc=False)

    def _attach_container_secret(self, container_secrets, container_id,
                                 session):
        if container_secrets:
            for container_secret in container_secrets:
                container_secret.container_id = container_id
                if container_secret.id is None:
                    self.container_secrets.append(container_secret.create(
                        session=session))
                else:
                    self.container_secrets.append(container_secret.save(
                        session=session))

    def _attach_consumers(self, consumers, container_id, session):
        if consumers:
            for consumer in consumers:
                consumer.container_id = container_id
                if consumer.id is None:
                    self.consumers.append(consumer.create(session=session))
                else:
                    self.consumers.append(consumer.save(session=session))

    def create(self, session=None):
        fields = self.obj_get_changes()
        super(Container, self).create(session=session)
        if 'container_secrets' in fields:
            self._attach_container_secret(
                fields['container_secrets'],
                container_id=self.id, session=session)
        if 'consumers' in fields:
            self._attach_consumers(fields['consumers'],
                                   container_id=self.id, session=session)

    def save(self, session=None):
        fields = self.obj_get_changes()
        super(Container, self).save(session=session)
        if 'consumers' in fields:
            self._attach_consumers(fields['consumers'],
                                   container_id=self.id, session=session)

    @classmethod
    def get_by_create_date(cls, external_project_id, offset_arg=None,
                           limit_arg=None, name_arg=None,
                           suppress_exception=False, session=None):
        entities_db, offset, limit, total = cls.db_repo.get_by_create_date(
            external_project_id, offset_arg, limit_arg, name_arg,
            suppress_exception, session
        )
        entities_obj = [cls()._from_db_object(entity_db)
                        for entity_db in entities_db]
        return entities_obj, offset, limit, total

    @classmethod
    def get_container_by_id(cls, entity_id, suppress_exception=False,
                            session=None):
        entity_db = cls.db_repo.get_container_by_id(entity_id,
                                                    suppress_exception,
                                                    session)
        return cls()._from_db_object(entity_db)



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.
"""
API JSON validators.
"""

import abc
import base64
import re

import jsonschema as schema
from ldap3.core import exceptions as ldap_exceptions
from ldap3.utils.dn import parse_dn
from OpenSSL import crypto
from oslo_utils import timeutils

from barbican.api import controllers
from barbican.common import config
from barbican.common import exception
from barbican.common import hrefs
from barbican.common import utils
from barbican import i18n as u
from barbican.model import models
from barbican.model import repositories as repo
from barbican.plugin.interface import secret_store
from barbican.plugin.util import mime_types


DEFAULT_MAX_SECRET_BYTES = config.DEFAULT_MAX_SECRET_BYTES
LOG = utils.getLogger(__name__)
CONF = config.CONF

MYSQL_SMALL_INT_MAX = 32767

ACL_OPERATIONS = ['read', 'write', 'delete', 'list']


def secret_too_big(data):
    if isinstance(data, str):
        return len(data.encode('UTF-8')) > CONF.max_allowed_secret_in_bytes
    else:
        return len(data) > CONF.max_allowed_secret_in_bytes


def get_invalid_property(validation_error):
    # we are interested in the second item which is the failed propertyName.
    if validation_error.schema_path and len(validation_error.schema_path) > 1:
        return validation_error.schema_path[1]


def validate_stored_key_rsa_container(project_id, container_ref, req):
    try:
        container_id = hrefs.get_container_id_from_ref(container_ref)
    except Exception:
        reason = u._("Bad Container Reference {ref}").format(
            ref=container_ref
        )
        raise exception.InvalidContainer(reason=reason)

    container_repo = repo.get_container_repository()

    container = container_repo.get_container_by_id(entity_id=container_id,
                                                   suppress_exception=True)
    if not container:
        reason = u._("Container Not Found")
        raise exception.InvalidContainer(reason=reason)

    if container.type != 'rsa':
        reason = u._("Container Wrong Type")
        raise exception.InvalidContainer(reason=reason)

    ctxt = controllers._get_barbican_context(req)
    inst = controllers.containers.ContainerController(container)
    controllers._do_enforce_rbac(inst, req,
                                 controllers.containers.CONTAINER_GET,
                                 ctxt)


class ValidatorBase(object, metaclass=abc.ABCMeta):
    """Base class for validators."""

    name = ''

    @abc.abstractmethod
    def validate(self, json_data, parent_schema=None):
        """Validate the input JSON.

        :param json_data: JSON to validate against this class' internal schema.
        :param parent_schema: Name of the parent schema to this schema.
        :returns: dict -- JSON content, post-validation and
        :                 normalization/defaulting.
        :raises: schema.ValidationError on schema violations.

        """

    def _full_name(self, parent_schema=None):
        """Validator schema name accessor

        Returns the full schema name for this validator,
        including parent name.
        """
        schema_name = self.name
        if parent_schema:
            schema_name = u._(
                "{schema_name}' within '{parent_schema_name}").format(
                    schema_name=self.name,
                    parent_schema_name=parent_schema)
        return schema_name

    def _assert_schema_is_valid(self, json_data, schema_name):
        """Assert that the JSON structure is valid for the given schema.

        :raises: InvalidObject exception if the data is not schema compliant.
        """
        try:
            schema.validate(json_data, self.schema)
        except schema.ValidationError as e:
            raise exception.InvalidObject(schema=schema_name,
                                          reason=e.message,
                                          property=get_invalid_property(e))

    def _assert_validity(self, valid_condition, schema_name, message,
                         property):
        """Assert that a certain condition is met.

        :raises: InvalidObject exception if the condition is not met.
        """
        if not valid_condition:
            raise exception.InvalidObject(schema=schema_name, reason=message,
                                          property=property)


class NewSecretValidator(ValidatorBase):
    """Validate a new secret."""

    def __init__(self):
        self.name = 'Secret'

        # TODO(jfwood): Get the list of mime_types from the crypto plugins?
        self.schema = {
            "type": "object",
            "properties": {
                "name": {"type": ["string", "null"], "maxLength": 255},
                "algorithm": {"type": "string", "maxLength": 255},
                "mode": {"type": "string", "maxLength": 255},
                "bit_length": {
                    "type": "integer",
                    "minimum": 1,
                    "maximum": MYSQL_SMALL_INT_MAX
                },
                "expiration": {"type": "string", "maxLength": 255},
                "payload": {"type": "string"},
                "secret_type": {
                    "type": "string",
                    "maxLength": 80,
                    "enum": [secret_store.SecretType.SYMMETRIC,
                             secret_store.SecretType.PASSPHRASE,
                             secret_store.SecretType.PRIVATE,
                             secret_store.SecretType.PUBLIC,
                             secret_store.SecretType.CERTIFICATE,
                             secret_store.SecretType.OPAQUE]
                },
                "payload_content_type": {
                    "type": ["string", "null"],
                    "maxLength": 255
                },
                "payload_content_encoding": {
                    "type": "string",
                    "maxLength": 255,
                    "enum": [
                        "base64"
                    ]
                },
                "transport_key_needed": {
                    "type": "string",
                    "enum": ["true", "false"]
                },
                "transport_key_id": {"type": "string"},
            },
        }

    def validate(self, json_data, parent_schema=None):
        """Validate the input JSON for the schema for secrets."""
        schema_name = self._full_name(parent_schema)
        self._assert_schema_is_valid(json_data, schema_name)

        json_data['name'] = self._extract_name(json_data)

        expiration = self._extract_expiration(json_data, schema_name)
        self._assert_expiration_is_valid(expiration, schema_name)
        json_data['expiration'] = expiration
        content_type = json_data.get('payload_content_type')

        if 'payload' in json_data:
            content_encoding = json_data.get('payload_content_encoding')
            self._validate_content_parameters(content_type, content_encoding,
                                              schema_name)

            payload = self._extract_payload(json_data)
            self._assert_validity(payload, schema_name,
                                  u._("If 'payload' specified, must be non "
                                      "empty"),
                                  "payload")
            self._validate_payload_by_content_encoding(content_encoding,
                                                       payload, schema_name)
            json_data['payload'] = payload
        elif 'payload_content_type' in json_data:
            # parent_schema would be populated if it comes from an order.
            self._assert_validity(parent_schema is not None, schema_name,
                                  u._("payload must be provided when "
                                      "payload_content_type is specified"),
                                  "payload")

            if content_type:
                self._assert_validity(
                    mime_types.is_supported(content_type),
                    schema_name,
                    u._("payload_content_type is not one of {supported}"
                        ).format(supported=mime_types.SUPPORTED),
                    "payload_content_type")

        return json_data

    def _extract_name(self, json_data):
        """Extracts and returns the name from the JSON data."""
        name = json_data.get('name')
        if isinstance(name, str):
            return name.strip()
        return None

    def _extract_expiration(self, json_data, schema_name):
        """Extracts and returns the expiration date from the JSON data."""
        expiration = None
        expiration_raw = json_data.get('expiration')
        if expiration_raw and expiration_raw.strip():
            try:
                expiration_tz = timeutils.parse_isotime(expiration_raw.strip())
                expiration = timeutils.normalize_time(expiration_tz)
            except ValueError:
                LOG.exception("Problem parsing expiration date")
                raise exception.InvalidObject(
                    schema=schema_name,
                    reason=u._("Invalid date for 'expiration'"),
                    property="expiration")

        return expiration

    def _assert_expiration_is_valid(self, expiration, schema_name):
        """Asserts that the given expiration date is valid.

        Expiration dates must be in the future, not the past.
        """
        if expiration:
            # Verify not already expired.
            utcnow = timeutils.utcnow()
            self._assert_validity(expiration > utcnow, schema_name,
                                  u._("'expiration' is before current time"),
                                  "expiration")

    def _validate_content_parameters(self, content_type, content_encoding,
                                     schema_name):
        """Content parameter validator.

        Check that the content_type, content_encoding and the parameters
        that they affect are valid.
        """
        self._assert_validity(
            content_type is not None,
            schema_name,
            u._("If 'payload' is supplied, 'payload_content_type' must also "
                "be supplied."),
            "payload_content_type")

        self._assert_validity(
            mime_types.is_supported(content_type),
            schema_name,
            u._("payload_content_type is not one of {supported}"
                ).format(supported=mime_types.SUPPORTED),
            "payload_content_type")

        self._assert_validity(
            mime_types.is_content_type_with_encoding_supported(
                content_type,
                content_encoding),
            schema_name,
            u._("payload_content_encoding is not one of {supported}").format(
                supported=mime_types.get_supported_encodings(content_type)),
            "payload_content_encoding")

    def _validate_payload_by_content_encoding(self, payload_content_encoding,
                                              payload, schema_name):
        if payload_content_encoding == 'base64':
            try:
                base64.b64decode(payload)
            except Exception:
                LOG.exception("Problem parsing payload")
                raise exception.InvalidObject(
                    schema=schema_name,
                    reason=u._("Invalid payload for payload_content_encoding"),
                    property="payload")

    def _extract_payload(self, json_data):
        """Extracts and returns the payload from the JSON data.

        :raises: LimitExceeded if the payload is too big
        """
        payload = json_data.get('payload', '')
        if secret_too_big(payload):
            raise exception.LimitExceeded()

        return payload.strip()


class NewSecretMetadataValidator(ValidatorBase):
    """Validate new secret metadata."""

    def __init__(self):
        self.name = 'SecretMetadata'
        self.schema = {
            "type": "object",
            "$schema": "http://json-schema.org/draft-03/schema",
            "properties": {
                "metadata": {"type": "object", "required": True},
            }
        }

    def validate(self, json_data, parent_schema=None):
        """Validate the input JSON for the schema for secret metadata."""
        schema_name = self._full_name(parent_schema)
        self._assert_schema_is_valid(json_data, schema_name)
        return self._extract_metadata(json_data)

    def _extract_metadata(self, json_data):
        """Extracts and returns the metadata from the JSON data."""
        metadata = json_data['metadata']

        for key in list(metadata):
            # make sure key is a string and url-safe.
            if not isinstance(key, str):
                raise exception.InvalidMetadataRequest()
            self._check_string_url_safe(key)

            # make sure value is a string.
            value = metadata[key]
            if not isinstance(value, str):
                raise exception.InvalidMetadataRequest()

            # If key is not lowercase, then change it
            if not key.islower():
                del metadata[key]
                metadata[key.lower()] = value

        return metadata

    def _check_string_url_safe(self, string):
        """Checks if string can be part of a URL."""
        if not re.match("^[A-Za-z0-9_-]*$", string):
            raise exception.InvalidMetadataKey()


class NewSecretMetadatumValidator(ValidatorBase):
    """Validate new secret metadatum."""

    def __init__(self):
        self.name = 'SecretMetadatum'
        self.schema = {
            "type": "object",
            "$schema": "http://json-schema.org/draft-03/schema",
            "properties": {
                "key": {
                    "type": "string",
                    "maxLength": 255,
                    "required": True
                },
                "value": {
                    "type": "string",
                    "maxLength": 255,
                    "required": True
                },
            },
            "additionalProperties": False
        }

    def validate(self, json_data, parent_schema=None):
        """Validate the input JSON for the schema for secret metadata."""
        schema_name = self._full_name(parent_schema)
        self._assert_schema_is_valid(json_data, schema_name)

        key = self._extract_key(json_data)
        value = self._extract_value(json_data)

        return {"key": key, "value": value}

    def _extract_key(self, json_data):
        """Extracts and returns the metadata from the JSON data."""
        key = json_data['key']
        self._check_string_url_safe(key)
        key = key.lower()
        return key

    def _extract_value(self, json_data):
        """Extracts and returns the metadata from the JSON data."""
        value = json_data['value']
        return value

    def _check_string_url_safe(self, string):
        """Checks if string can be part of a URL."""
        if not re.match("^[A-Za-z0-9_-]*$", string):
            raise exception.InvalidMetadataKey()


class CACommonHelpersMixin(object):
    def _validate_subject_dn_data(self, subject_dn):
        """Confirm that the subject_dn contains valid data

        Validate that the subject_dn string parses without error
        If not, raise InvalidSubjectDN
        """
        try:
            parse_dn(subject_dn)
        except ldap_exceptions.LDAPInvalidDnError:
            raise exception.InvalidSubjectDN(subject_dn=subject_dn)


# TODO(atiwari) - Split this validator module and unit tests
# into smaller modules
class TypeOrderValidator(ValidatorBase, CACommonHelpersMixin):
    """Validate a new typed order."""

    def __init__(self):
        self.name = 'Order'
        self.schema = {
            "type": "object",
            "$schema": "http://json-schema.org/draft-03/schema",
            "properties": {
                "meta": {
                    "type": "object",
                    "required": True
                },
                "type": {
                    "type": "string",
                    "required": True,
                    "enum": ['key', 'asymmetric', 'certificate']
                }
            }
        }

    def validate(self, json_data, parent_schema=None):
        schema_name = self._full_name(parent_schema)

        self._assert_schema_is_valid(json_data, schema_name)

        order_type = json_data.get('type').lower()

        if order_type == models.OrderType.CERTIFICATE:
            certificate_meta = json_data.get('meta')
            self._validate_certificate_meta(certificate_meta, schema_name)

        elif order_type == models.OrderType.ASYMMETRIC:
            asymmetric_meta = json_data.get('meta')
            self._validate_asymmetric_meta(asymmetric_meta, schema_name)

        elif order_type == models.OrderType.KEY:
            key_meta = json_data.get('meta')
            self._validate_key_meta(key_meta, schema_name)

        else:
            self._raise_feature_not_implemented(order_type, schema_name)

        return json_data

    def _validate_key_meta(self, key_meta, schema_name):
        """Validation specific to meta for key type order."""

        secret_validator = NewSecretValidator()
        secret_validator.validate(key_meta, parent_schema=self.name)

        self._assert_validity(key_meta.get('payload') is None,
                              schema_name,
                              u._("'payload' not allowed "
                                  "for key type order"), "meta")

        # Validation secret generation related fields.
        # TODO(jfwood): Invoke the crypto plugin for this purpose

        self._validate_meta_parameters(key_meta, "key", schema_name)

    def _validate_asymmetric_meta(self, asymmetric_meta, schema_name):
        """Validation specific to meta for asymmetric type order."""

        # Validate secret metadata.
        secret_validator = NewSecretValidator()
        secret_validator.validate(asymmetric_meta, parent_schema=self.name)

        self._assert_validity(asymmetric_meta.get('payload') is None,
                              schema_name,
                              u._("'payload' not allowed "
                                  "for asymmetric type order"), "meta")

        self._validate_meta_parameters(asymmetric_meta, "asymmetric key",
                                       schema_name)

    def _get_required_metadata_value(self, metadata, key):
        data = metadata.get(key, None)
        if data is None:
            raise exception.MissingMetadataField(required=key)
        return data

    def _validate_certificate_meta(self, certificate_meta, schema_name):
        """Validation specific to meta for certificate type order."""

        self._assert_validity(certificate_meta.get('payload') is None,
                              schema_name,
                              u._("'payload' not allowed "
                                  "for certificate type order"), "meta")

        if 'profile' in certificate_meta:
            if 'ca_id' not in certificate_meta:
                raise exception.MissingMetadataField(required='ca_id')

        jump_table = {
            'simple-cmc': self._validate_simple_cmc_request,
            'full-cmc': self._validate_full_cmc_request,
            'stored-key': self._validate_stored_key_request,
            'custom': self._validate_custom_request
        }

        request_type = certificate_meta.get("request_type", "custom")
        if request_type not in jump_table:
            raise exception.InvalidCertificateRequestType(request_type)

        jump_table[request_type](certificate_meta)

    def _validate_simple_cmc_request(self, certificate_meta):
        """Validates simple CMC (which are PKCS10 requests)."""
        request_data = self._get_required_metadata_value(
            certificate_meta, "request_data")
        self._validate_pkcs10_data(request_data)

    def _validate_full_cmc_request(self, certificate_meta):
        """Validate full CMC request.

        :param certificate_meta: request data from the order
        :raises: FullCMCNotSupported
        """
        raise exception.FullCMCNotSupported()

    def _validate_stored_key_request(self, certificate_meta):
        """Validate stored-key cert request."""
        self._get_required_metadata_value(
            certificate_meta, "container_ref")
        subject_dn = self._get_required_metadata_value(
            certificate_meta, "subject_dn")
        self._validate_subject_dn_data(subject_dn)
        # container will be validated by validate_stored_key_rsa_container()

        extensions = certificate_meta.get("extensions", None)
        if extensions:
            self._validate_extensions_data(extensions)

    def _validate_custom_request(self, certificate_meta):
        """Validate custom data request

        We cannot do any validation here because the request
        parameters are custom.  Validation will be done by the
        plugin.  We may choose to select the relevant plugin and
        call the supports() method to raise validation errors.
        """
        pass

    def _validate_pkcs10_data(self, request_data):
        """Confirm that the request_data is valid base64 encoded PKCS#10.

        Base64 decode the request, if it fails raise PayloadDecodingError.
        Then parse data into the ASN.1 structure defined by PKCS10 and
        verify the signing information.
        If parsing of verifying fails, raise InvalidPKCS10Data.
        """
        try:
            csr_pem = base64.b64decode(request_data)
        except Exception:
            raise exception.PayloadDecodingError()

        try:
            csr = crypto.load_certificate_request(crypto.FILETYPE_PEM,
                                                  csr_pem)
        except Exception:
            reason = u._("Bad format")
            raise exception.InvalidPKCS10Data(reason=reason)

        try:
            pubkey = csr.get_pubkey()
            csr.verify(pubkey)
        except Exception:
            reason = u._("Signing key incorrect")
            raise exception.InvalidPKCS10Data(reason=reason)

    def _validate_full_cmc_data(self, request_data):
        """Confirm that request_data is valid Full CMC data."""
        """
        TODO(alee-3) complete this function

        Parse data into the ASN.1 structure defined for full CMC.
        If parsing fails, raise InvalidCMCData
        """
        pass

    def _validate_extensions_data(self, extensions):
        """Confirm that the extensions data is valid.

        :param extensions: base 64 encoded ASN.1 string of extension data
        :raises: CertificateExtensionsNotSupported
        """
        """
        TODO(alee-3) complete this function

        Parse the extensions data into the correct ASN.1 structure.
        If the parsing fails, throw InvalidExtensionsData.

        For now, fail this validation because extensions parsing is not
        supported.
        """
        raise exception.CertificateExtensionsNotSupported()

    def _validate_meta_parameters(self, meta, order_type, schema_name):
        self._assert_validity(meta.get('algorithm'),
                              schema_name,
                              u._("'algorithm' is required field "
                                  "for {0} type order").format(order_type),
                              "meta")

        self._assert_validity(meta.get('bit_length'),
                              schema_name,
                              u._("'bit_length' is required field "
                                  "for {0} type order").format(order_type),
                              "meta")

        self._validate_bit_length(meta, schema_name)

    def _extract_expiration(self, json_data, schema_name):
        """Extracts and returns the expiration date from the JSON data."""
        expiration = None
        expiration_raw = json_data.get('expiration', None)
        if expiration_raw and expiration_raw.strip():
            try:
                expiration_tz = timeutils.parse_isotime(expiration_raw)
                expiration = timeutils.normalize_time(expiration_tz)
            except ValueError:
                LOG.exception("Problem parsing expiration date")
                raise exception.InvalidObject(schema=schema_name,
                                              reason=u._("Invalid date "
                                                         "for 'expiration'"),
                                              property="expiration")

        return expiration

    def _validate_bit_length(self, meta, schema_name):

        bit_length = int(meta.get('bit_length'))
        if bit_length % 8 != 0:
            raise exception.UnsupportedField(field="bit_length",
                                             schema=schema_name,
                                             reason=u._("Must be a"
                                                        " positive integer"
                                                        " that is a"
                                                        " multiple of 8"))

    def _raise_feature_not_implemented(self, order_type, schema_name):
        raise exception.FeatureNotImplemented(field='type',
                                              schema=schema_name,
                                              reason=u._("Feature not "
                                                         "implemented for "
                                                         "'{0}' order type")
                                                    .format(order_type))


class ACLValidator(ValidatorBase):
    """Validate ACL(s)."""

    def __init__(self):
        self.name = 'ACL'

        self.schema = {
            "$schema": "http://json-schema.org/draft-04/schema#",
            "definitions": {
                "acl_defintion": {
                    "type": "object",
                    "properties": {
                        "users": {
                            "type": "array",
                            "items": [
                                {"type": "string", "maxLength": 255}
                            ]
                        },
                        "project-access": {"type": "boolean"}
                    },
                    "additionalProperties": False
                }
            },
            "type": "object",
            "properties": {
                "read": {"$ref": "#/definitions/acl_defintion"},
            },
            "additionalProperties": False
        }

    def validate(self, json_data, parent_schema=None):
        schema_name = self._full_name(parent_schema)

        self._assert_schema_is_valid(json_data, schema_name)
        return json_data


class ContainerConsumerValidator(ValidatorBase):
    """Validate a Consumer."""

    def __init__(self):
        self.name = 'Consumer'
        self.schema = {
            "type": "object",
            "properties": {
                "URL": {"type": "string", "maxLength": 255, "minLength": 1},
                "name": {"type": "string", "maxLength": 36, "minLength": 1}
            },
            "required": ["name", "URL"]
        }

    def validate(self, json_data, parent_schema=None):
        schema_name = self._full_name(parent_schema)

        self._assert_schema_is_valid(json_data, schema_name)
        return json_data


class ContainerSecretValidator(ValidatorBase):
    """Validate a Container Secret."""

    def __init__(self):
        self.name = 'ContainerSecret'
        self.schema = {
            "type": "object",
            "properties": {
                "name": {"type": "string", "maxLength": 255},
                "secret_ref": {"type": "string", "minLength": 1}
            },
            "required": ["secret_ref"]
        }

    def validate(self, json_data, parent_schema=None):
        schema_name = self._full_name(parent_schema)

        self._assert_schema_is_valid(json_data, schema_name)
        return json_data


class ContainerValidator(ValidatorBase):
    """Validator for all types of Container."""

    def __init__(self):
        self.name = 'Container'
        self.schema = {
            "type": "object",
            "properties": {
                "name": {"type": ["string", "null"], "maxLength": 255},
                "type": {
                    "type": "string",
                    # TODO(hgedikli): move this to a common location
                    "enum": ["generic", "rsa", "certificate"]
                },
                "secret_refs": {
                    "type": "array",
                    "items": {
                        "type": "object",
                        "required": ["secret_ref"],
                        "properties": {
                            "name": {
                                "type": ["string", "null"], "maxLength": 255
                            },
                            "secret_ref": {"type": "string", "minLength": 1}
                        }
                    }
                }
            },
            "required": ["type"]
        }

    def validate(self, json_data, parent_schema=None):
        schema_name = self._full_name(parent_schema)

        self._assert_schema_is_valid(json_data, schema_name)

        container_type = json_data.get('type')
        secret_refs = json_data.get('secret_refs')

        if not secret_refs:
            return json_data

        secret_refs_names = set(secret_ref.get('name', '')
                                for secret_ref in secret_refs)

        self._assert_validity(
            len(secret_refs_names) == len(secret_refs),
            schema_name,
            u._("Duplicate reference names are not allowed"),
            "secret_refs")

        # The combination of container_id and secret_id is expected to be
        # primary key for container_secret so same secret id (ref) cannot be
        # used within a container
        secret_ids = set(self._get_secret_id_from_ref(secret_ref)
                         for secret_ref in secret_refs)

        self._assert_validity(
            len(secret_ids) == len(secret_refs),
            schema_name,
            u._("Duplicate secret ids are not allowed"),
            "secret_refs")

        # Ensure that our secret refs are valid relative to our config, no
        # spoofing allowed!
        req_host_href = utils.get_base_url_from_request()
        for secret_ref in secret_refs:
            if not secret_ref.get('secret_ref').startswith(req_host_href):
                raise exception.UnsupportedField(
                    field='secret_ref',
                    schema=schema_name,
                    reason=u._(
                        "Secret_ref does not match the configured hostname, "
                        "please try again"
                    )
                )

        if container_type == 'rsa':
            self._validate_rsa(secret_refs_names, schema_name)
        elif container_type == 'certificate':
            self._validate_certificate(secret_refs_names, schema_name)

        return json_data

    def _validate_rsa(self, secret_refs_names, schema_name):
        required_names = {'public_key', 'private_key'}
        optional_names = {'private_key_passphrase'}
        contains_unsupported_names = self._contains_unsupported_names(
            secret_refs_names, required_names | optional_names)
        self._assert_validity(
            not contains_unsupported_names,
            schema_name,
            u._("only 'private_key', 'public_key' and "
                "'private_key_passphrase' reference names are "
                "allowed for RSA type"),
            "secret_refs")

        self._assert_validity(
            self._has_minimum_required(secret_refs_names, required_names),
            schema_name,
            u._("The minimum required reference names are 'public_key' and"
                "'private_key' for RSA type"),
            "secret_refs")

    def _validate_certificate(self, secret_refs_names, schema_name):
        required_names = {'certificate'}
        optional_names = {'private_key', 'private_key_passphrase',
                          'intermediates'}
        contains_unsupported_names = self._contains_unsupported_names(
            secret_refs_names, required_names.union(optional_names))
        self._assert_validity(
            not contains_unsupported_names,
            schema_name,
            u._("only 'private_key', 'certificate' , "
                "'private_key_passphrase',  or 'intermediates' "
                "reference names are allowed for Certificate type"),
            "secret_refs")

        self._assert_validity(
            self._has_minimum_required(secret_refs_names, required_names),
            schema_name,
            u._("The minimum required reference name is 'certificate' "
                "for Certificate type"),
            "secret_refs")

    def _contains_unsupported_names(self, secret_refs_names, supported_names):
        if secret_refs_names.difference(supported_names):
            return True
        return False

    def _has_minimum_required(self, secret_refs_names, required_names):
        if required_names.issubset(secret_refs_names):
            return True
        return False

    def _get_secret_id_from_ref(self, secret_ref):
        secret_id = secret_ref.get('secret_ref')
        if secret_id.endswith('/'):
            secret_id = secret_id.rsplit('/', 2)[1]
        elif '/' in secret_id:
            secret_id = secret_id.rsplit('/', 1)[1]

        return secret_id


class NewTransportKeyValidator(ValidatorBase):
    """Validate a new transport key."""

    def __init__(self):
        self.name = 'Transport Key'

        self.schema = {
            "type": "object",
            "properties": {
                "plugin_name": {"type": "string"},
                "transport_key": {"type": "string"},
            },
        }

    def validate(self, json_data, parent_schema=None):
        schema_name = self._full_name(parent_schema)

        self._assert_schema_is_valid(json_data, schema_name)

        plugin_name = json_data.get('plugin_name', '').strip()
        self._assert_validity(plugin_name,
                              schema_name,
                              u._("plugin_name must be provided"),
                              "plugin_name")
        json_data['plugin_name'] = plugin_name

        transport_key = json_data.get('transport_key', '').strip()
        self._assert_validity(transport_key,
                              schema_name,
                              u._("transport_key must be provided"),
                              "transport_key")
        json_data['transport_key'] = transport_key

        return json_data


class ProjectQuotaValidator(ValidatorBase):
    """Validate a new project quota."""

    def __init__(self):
        self.name = 'Project Quota'

        self.schema = {
            'type': 'object',
            'properties': {
                'project_quotas': {
                    'type': 'object',
                    'properties': {
                        'secrets': {'type': 'integer'},
                        'orders': {'type': 'integer'},
                        'containers': {'type': 'integer'},
                        'consumers': {'type': 'integer'},
                        'cas': {'type': 'integer'}
                    },
                    'additionalProperties': False,
                }
            },
            'required': ['project_quotas'],
            'additionalProperties': False
        }

    def validate(self, json_data, parent_schema=None):
        schema_name = self._full_name(parent_schema)

        self._assert_schema_is_valid(json_data, schema_name)

        return json_data


class NewCAValidator(ValidatorBase, CACommonHelpersMixin):
    """Validate new CA(s)."""

    def __init__(self):
        self.name = 'CA'

        self.schema = {
            'type': 'object',
            'properties': {
                'name': {'type': 'string', "minLength": 1},
                'subject_dn': {'type': 'string', "minLength": 1},
                'parent_ca_ref': {'type': 'string', "minLength": 1},
                'description': {'type': 'string'},
            },
            'required': ['name', 'subject_dn', 'parent_ca_ref'],
            'additionalProperties': False
        }

    def validate(self, json_data, parent_schema=None):
        schema_name = self._full_name(parent_schema)

        self._assert_schema_is_valid(json_data, schema_name)

        subject_dn = json_data['subject_dn']
        self._validate_subject_dn_data(subject_dn)
        return json_data


class SecretConsumerValidator(ValidatorBase):
    """Validate a new Secret Consumer."""

    def __init__(self):
        self.name = "Secret Consumer"

        self.schema = {
            "type": "object",
            "properties": {
                "service": {
                    "type": "string",
                    "maxLength": 255,
                    "minLength": 1,
                },
                "resource_type": {
                    "type": "string",
                    "maxLength": 255,
                    "minLength": 1,
                },
                "resource_id": {"type": "string", "minLength": 1},
            },
            "required": ["service", "resource_type", "resource_id"],
        }

    def validate(self, json_data, parent_schema=None):
        schema_name = self._full_name(parent_schema)

        self._assert_schema_is_valid(json_data, schema_name)

        return json_data
